@@ -2,7 +2,8 @@ import copy
 from argparse import Namespace
 from typing import Dict, List, Optional, Tuple, Union
 
-from jina import (
+from jina.constants import (
+    __default_composite_gateway__,
     __default_executor__,
     __default_grpc_gateway__,
     __default_http_gateway__,
@@ -10,7 +11,7 @@ from jina import (
 )
 from jina.enums import PodRoleType
 from jina.excepts import NoContainerizedError
-from jina.orchestrate.deployments import BaseDeployment
+from jina.orchestrate.deployments import Deployment
 from jina.orchestrate.deployments.config.helper import (
     construct_runtime_container_args,
     get_base_executor_version,
@@ -38,8 +39,6 @@ class K8sDeploymentConfig:
             common_args: Union['Namespace', Dict],
             deployment_args: Union['Namespace', Dict],
             k8s_namespace: str,
-            k8s_deployments_addresses: Optional[Dict[str, List[str]]] = None,
-            k8s_deployments_metadata: Optional[Dict[str, Dict[str, str]]] = None,
         ):
             self.name = name
             self.dns_name = to_compatible_name(name)
@@ -51,14 +50,11 @@ class K8sDeploymentConfig:
             self.deployment_args = deployment_args
             self.num_replicas = getattr(self.deployment_args, 'replicas', 1)
             self.k8s_namespace = k8s_namespace
-            self.k8s_deployments_addresses = k8s_deployments_addresses
-            self.k8s_deployments_metadata = k8s_deployments_metadata
 
         def get_gateway_yamls(
             self,
         ) -> List[Dict]:
             cargs = copy.copy(self.deployment_args)
-            cargs.deployments_addresses = self.k8s_deployments_addresses
             from jina.helper import ArgNamespace
             from jina.parsers import set_gateway_parser
 
@@ -69,7 +65,6 @@ class K8sDeploymentConfig:
                 'uses_after',
                 'workspace',
                 'workspace_id',
-                'upload_files',
                 'noblock_on_start',
                 'env',
             }
@@ -79,6 +74,7 @@ class K8sDeploymentConfig:
                 __default_http_gateway__,
                 __default_websocket_gateway__,
                 __default_grpc_gateway__,
+                __default_composite_gateway__,
             ]:
                 cargs.uses = 'config.yml'
 
@@ -102,6 +98,7 @@ class K8sDeploymentConfig:
                 monitoring=self.common_args.monitoring,
                 port_monitoring=self.common_args.port_monitoring,
                 protocol=self.common_args.protocol,
+                timeout_ready=self.common_args.timeout_ready,
             )
 
         def _get_image_name(self, uses: Optional[str]):
@@ -115,6 +112,7 @@ class K8sDeploymentConfig:
                 __default_http_gateway__,
                 __default_websocket_gateway__,
                 __default_grpc_gateway__,
+                __default_composite_gateway__,
             ]:
                 image_name = get_image_name(uses)
 
@@ -130,7 +128,7 @@ class K8sDeploymentConfig:
             )
 
         def get_runtime_yamls(
-            self,
+            self, k8s_port: Optional[int] = GrpcConnectionPool.K8S_PORT
         ) -> List[Dict]:
             cargs = copy.copy(self.deployment_args)
 
@@ -206,18 +204,20 @@ class K8sDeploymentConfig:
                 pod_type=self.pod_type,
                 shard_id=self.shard_id,
                 env=cargs.env,
+                env_from_secret=cargs.env_from_secret,
                 gpus=cargs.gpus if hasattr(cargs, 'gpus') else None,
                 monitoring=cargs.monitoring,
                 port_monitoring=cargs.port_monitoring,
                 volumes=getattr(cargs, 'volumes', None),
+                timeout_ready=cargs.timeout_ready,
+                k8s_port=k8s_port,
             )
 
     def __init__(
         self,
         args: Union['Namespace', Dict],
         k8s_namespace: Optional[str] = None,
-        k8s_deployments_addresses: Optional[Dict[str, List[str]]] = None,
-        k8s_deployments_metadata: Optional[Dict[str, Dict[str, str]]] = None,
+        k8s_port: Optional[int] = GrpcConnectionPool.K8S_PORT,
     ):
         # External Deployments should be ignored in a K8s based Flow
         assert not (hasattr(args, 'external') and args.external)
@@ -227,8 +227,7 @@ class K8sDeploymentConfig:
                 'You need to use a containerized Executor. You may check `jina hub --help` to see how Jina Hub can help you building containerized Executors.'
             )
         self.k8s_namespace = k8s_namespace
-        self.k8s_deployments_addresses = k8s_deployments_addresses
-        self.k8s_deployments_metadata = k8s_deployments_metadata
+        self.k8s_port = k8s_port
         self.head_deployment = None
         self.args = copy.copy(args)
         if k8s_namespace is not None:
@@ -236,7 +235,7 @@ class K8sDeploymentConfig:
             self.args.k8s_namespace = k8s_namespace
         self.name = self.args.name
 
-        self.deployment_args = self._get_deployment_args(self.args)
+        self.deployment_args = self._get_deployment_args(self.args, k8s_port=k8s_port)
 
         if self.deployment_args['head_deployment'] is not None:
             self.head_deployment = self._K8sDeployment(
@@ -248,7 +247,6 @@ class K8sDeploymentConfig:
                 deployment_args=self.deployment_args['head_deployment'],
                 pod_type=PodRoleType.HEAD,
                 k8s_namespace=self.k8s_namespace,
-                k8s_deployments_addresses=self.k8s_deployments_addresses,
             )
         self.worker_deployments = []
         deployment_args = self.deployment_args['deployments']
@@ -266,16 +264,12 @@ class K8sDeploymentConfig:
                     else PodRoleType.GATEWAY,
                     jina_deployment_name=self.name,
                     k8s_namespace=self.k8s_namespace,
-                    k8s_deployments_addresses=self.k8s_deployments_addresses
-                    if name == 'gateway'
-                    else None,
-                    k8s_deployments_metadata=self.k8s_deployments_metadata
-                    if name == 'gateway'
-                    else None,
                 )
             )
 
-    def _get_deployment_args(self, args):
+    def _get_deployment_args(
+        self, args, k8s_port: Optional[int] = GrpcConnectionPool.K8S_PORT
+    ):
         parsed_args = {
             'head_deployment': None,
             'deployments': [],
@@ -287,11 +281,11 @@ class K8sDeploymentConfig:
         if args.name != 'gateway':
             # head deployment only exists for sharded deployments
             if shards > 1:
-                parsed_args['head_deployment'] = BaseDeployment._copy_to_head_args(
+                parsed_args['head_deployment'] = Deployment._copy_to_head_args(
                     self.args
                 )
                 parsed_args['head_deployment'].gpus = None
-                parsed_args['head_deployment'].port = GrpcConnectionPool.K8S_PORT
+                parsed_args['head_deployment'].port = k8s_port
                 parsed_args[
                     'head_deployment'
                 ].port_monitoring = GrpcConnectionPool.K8S_PORT_MONITORING
@@ -310,7 +304,7 @@ class K8sDeploymentConfig:
                     )
                     connection_list[
                         str(i)
-                    ] = f'{name}.{self.k8s_namespace}.svc:{GrpcConnectionPool.K8S_PORT}'
+                    ] = f'{name}.{self.k8s_namespace}.svc:{k8s_port}'
 
                 parsed_args['head_deployment'].connection_list = json.dumps(
                     connection_list
@@ -331,11 +325,12 @@ class K8sDeploymentConfig:
 
         for i in range(shards):
             cargs = copy.deepcopy(args)
+            cargs.host = args.host[0]
             cargs.shard_id = i
             cargs.uses_before = None
             cargs.uses_after = None
             if args.name != 'gateway':
-                cargs.port = GrpcConnectionPool.K8S_PORT
+                cargs.port = k8s_port
                 cargs.port_monitoring = GrpcConnectionPool.K8S_PORT_MONITORING
 
             cargs.uses_before_address = None
@@ -371,7 +366,7 @@ class K8sDeploymentConfig:
             return [
                 (
                     deployment.dns_name,
-                    deployment.get_runtime_yamls(),
+                    deployment.get_runtime_yamls(k8s_port=self.k8s_port),
                 )
                 for deployment in deployments
             ]
