@@ -1,9 +1,10 @@
 import argparse
+import asyncio
 import json
 import os
 from abc import ABC
 from collections import defaultdict
-from typing import List
+from typing import TYPE_CHECKING, AsyncIterator, List
 
 import grpc
 from grpc_health.v1 import health, health_pb2, health_pb2_grpc
@@ -19,6 +20,9 @@ from jina.serve.runtimes.head.request_handling import HeaderRequestHandler
 from jina.serve.runtimes.helper import _get_grpc_server_options
 from jina.types.request.data import DataRequest, Response
 
+if TYPE_CHECKING:  # pragma: no cover
+    from jina.types.request import Request
+
 
 class HeadRuntime(AsyncNewLoopRuntime, ABC):
     """
@@ -133,6 +137,8 @@ class HeadRuntime(AsyncNewLoopRuntime, ABC):
             interceptors=self.aio_tracing_server_interceptors(),
         )
 
+        jina_pb2_grpc.add_JinaRPCServicer_to_server(self, self._grpc_server)
+
         jina_pb2_grpc.add_JinaSingleDataRequestRPCServicer_to_server(
             self, self._grpc_server
         )
@@ -142,6 +148,7 @@ class HeadRuntime(AsyncNewLoopRuntime, ABC):
         )
         jina_pb2_grpc.add_JinaInfoRPCServicer_to_server(self, self._grpc_server)
         service_names = (
+            jina_pb2.DESCRIPTOR.services_by_name['JinaRPC'].full_name,
             jina_pb2.DESCRIPTOR.services_by_name['JinaSingleDataRequestRPC'].full_name,
             jina_pb2.DESCRIPTOR.services_by_name['JinaDataRequestRPC'].full_name,
             jina_pb2.DESCRIPTOR.services_by_name['JinaDiscoverEndpointsRPC'].full_name,
@@ -159,23 +166,34 @@ class HeadRuntime(AsyncNewLoopRuntime, ABC):
             )
         reflection.enable_server_reflection(service_names, self._grpc_server)
 
-        bind_addr = f'0.0.0.0:{self.args.port}'
+        bind_addr = f'{self.args.host}:{self.args.port}'
         self._grpc_server.add_insecure_port(bind_addr)
         self.logger.debug(f'start listening on {bind_addr}')
         await self._grpc_server.start()
 
+    def _warmup(self):
+        self.warmup_task = asyncio.create_task(
+            self.request_handler.warmup(
+                connection_pool=self.connection_pool,
+                stop_event=self.warmup_stop_event,
+                deployment=self._deployment_name,
+            )
+        )
+
     async def async_run_forever(self):
         """Block until the GRPC server is terminated"""
+        self._warmup()
         await self._grpc_server.wait_for_termination()
 
     async def async_cancel(self):
         """Stop the GRPC server"""
         self.logger.debug('cancel HeadRuntime')
-
+        self.cancel_warmup_task()
         await self._grpc_server.stop(0)
 
     async def async_teardown(self):
         """Close the connection pool"""
+        self.cancel_warmup_task()
         await self._health_servicer.enter_graceful_shutdown()
         await self.async_cancel()
         await self.connection_pool.close()
@@ -200,6 +218,10 @@ class HeadRuntime(AsyncNewLoopRuntime, ABC):
             context.set_details(
                 f'|Head: Connection to worker (Executor) pod at address {err.dest_addr} could be established, but timed out.'
             )
+        elif err_code == grpc.StatusCode.NOT_FOUND:
+            context.set_details(
+                f'|Head: Connection to worker (Executor) pod at address {err.dest_addr} could be established, but resource was not found.'
+            )
         context.set_code(err.code())
         self.logger.error(f'Error while getting responses from Pods: {err.details()}')
         if err.request_id:
@@ -294,6 +316,7 @@ class HeadRuntime(AsyncNewLoopRuntime, ABC):
         :param context: grpc context
         :returns: the response request
         """
+        self.logger.debug('recv _status request')
         infoProto = jina_pb2.JinaInfoProto()
         version, env_info = get_full_version()
         for k, v in version.items():
@@ -301,3 +324,20 @@ class HeadRuntime(AsyncNewLoopRuntime, ABC):
         for k, v in env_info.items():
             infoProto.envs[k] = str(v)
         return infoProto
+
+    async def stream(
+        self, request_iterator, context=None, *args, **kwargs
+    ) -> AsyncIterator['Request']:
+        """
+        stream requests from client iterator and stream responses back.
+
+        :param request_iterator: iterator of requests
+        :param context: context of the grpc call
+        :param args: positional arguments
+        :param kwargs: keyword arguments
+        :yield: responses to the request
+        """
+        async for request in request_iterator:
+            yield await self.process_data([request], context)
+
+    Call = stream
